/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the reusable ccl java library
 * (http://www.kclee.com/clemens/java/ccl/).
 *
 * The Initial Developer of the Original Code is
 * Chr. Clemens Lee.
 * Portions created by Chr. Clemens Lee are Copyright (C) 2002
 * Chr. Clemens Lee. All Rights Reserved.
 *
 * Contributor(s): Chr. Clemens Lee <clemens@kclee.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

package ccl.swing;

import ccl.util.Util;

import java.util.Hashtable;


/**
 * A list that changes item color after item was selected.
 *
 * @version  $Id: HighlightableValue.java,v 1.4 2002/10/06 16:41:35 clemens Exp clemens $
 * @author   <a href="http://www.kclee.com/clemens/">
 *           Chr. Clemens Lee</a>
 *           &lt;<a href="mailto:clemens@kclee.com">
 *           clemens@kclee.com
 *           </a>>
 */
public class HighlightableValue implements Highlightable 
{
    private static final Object O_SHARE = new Object();
    private Hashtable _htHighlightedValues = null;
    private String _sKey = null;
    private String _pString = null;
    private boolean _bHighlighted = false;

    public HighlightableValue( Hashtable htHighlightedValues_,
                               String sKey_,
                               String pString_ ) 
    {
        super();

        Util.panicIf( pString_ == null );

        _htHighlightedValues = htHighlightedValues_;
        _sKey = sKey_;
        _pString = pString_;
        _bHighlighted = false;
        if ( _htHighlightedValues.get( sKey_ ) == O_SHARE ) 
        {
            _bHighlighted = true;
        }
    }

    public void setHighlightable( boolean bHighlightable_ ) 
    {
        if ( bHighlightable_ == _bHighlighted ) 
        {
            return;
        }

        _bHighlighted = bHighlightable_;

        if ( _bHighlighted ) 
        {
            _htHighlightedValues.put( _sKey, O_SHARE );
            Util.debug( "setHighlightable(..)._bHighlighted: " +
                        _bHighlighted );
        }
        else 
        {
            _htHighlightedValues.remove( _sKey );
        }
    }

    public boolean isHighlighted() 
    {
        return _bHighlighted;
    }

    public String toString() 
    {
        return _pString;
    }

    public boolean equals( Object pObject_ ) 
    {
        Util.debug( "HighlightableValue.equals(..).this: " + this );
        Util.debug( "HighlightableValue.equals(..).pObject_: " + pObject_ );
        return _pString.equals( pObject_ );
    }
}
