/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the reusable ccl java library
 * (http://www.kclee.com/clemens/java/ccl/).
 *
 * The Initial Developer of the Original Code is
 * Chr. Clemens Lee.
 * Portions created by Chr. Clemens Lee are Copyright (C) 2002
 * Chr. Clemens Lee. All Rights Reserved.
 *
 * Contributor(s): Chr. Clemens Lee <clemens@kclee.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

package ccl.swing;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.SwingUtil;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JPanel;

/**
 * Dialog with memory information.
 *
 * @version  $Id: MemoryDialog.java,v 1.9 2003/05/01 16:44:23 clemens Exp clemens $
 * @author   <a href="http://www.kclee.com/clemens/">
 *           Chr. Clemens Lee</a>
 *           &lt;<a href="mailto:clemens@kclee.com">
 *           clemens@kclee.com
 *           </a>>
 */
public class MemoryDialog extends ExitJFrame
                          implements ActionListener
{
    private MemoryPanel _pMemoryPanel = null;
    private JButton _btnOK = null;

    public MemoryDialog() 
    {
        this(new Frame());
    }
    
    public MemoryDialog( Frame frmParent_ ) 
    {
        super( "Memory" );
        
        setBackground( frmParent_.getBackground() );

        AutoGridBagLayout pAutoGridBagLayout = new AutoGridBagLayout();

        getContentPane().setLayout( pAutoGridBagLayout );
        ((JPanel)getContentPane()).setBorder( BorderFactory.
                                              createEmptyBorder( 5, 5, 5, 5 ) );

        _pMemoryPanel = new MemoryPanel();
        getContentPane().add( _pMemoryPanel );
        pAutoGridBagLayout.endLine();

        getContentPane().add( Box.createVerticalStrut( 2 ) );
        pAutoGridBagLayout.endLine();

        _btnOK = new JButton( "OK" );
        _btnOK.addActionListener( this );

        getContentPane().add( _btnOK );
        
        //FocusManager.setCurrentManager( new DefaultFocusManager() );

        //SwingUtil.setInitialFocus( this, _btnOK );
        addWindowListener( new WindowAdapter() {
                public void windowOpened( WindowEvent pWindowEvent_ ) 
                {
                    //SwingUtilities.invokeLater( new Runnable() {
                    //public void run() {
                    _btnOK.requestFocus();
                    //  }
                    //} );
                }
            } );

        pack();

        SwingUtil.centerComponent( this) ;
        show();

        _pMemoryPanel.start();
    }

    public void setVisible( boolean bVisible_ ) 
    {
        if ( bVisible_ ) 
        {
            //
        }
        else 
        {
            _pMemoryPanel.stop();
        }

        super.setVisible( bVisible_ );
    }

    public void actionPerformed(ActionEvent pActionEvent_) 
    {
        Object oSource = pActionEvent_.getSource();
        if (oSource == _btnOK) 
        {
            exit();
        }
    }
}
