/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the reusable ccl java library
 * (http://www.kclee.com/clemens/java/ccl/).
 *
 * The Initial Developer of the Original Code is
 * Chr. Clemens Lee.
 * Portions created by Chr. Clemens Lee are Copyright (C) 2002
 * Chr. Clemens Lee. All Rights Reserved.
 *
 * Contributor(s): Chr. Clemens Lee <clemens@kclee.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

package ccl.util.test;

import ccl.util.StreamCopier;
import ccl.util.Test;
import java.io.ByteArrayOutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;

/**
 * Test class to check if the StreamCopier class works
 * correctly.
 *
 * @version $Id: StreamCopierTest.java,v 1.2 2003/05/01 16:44:52 clemens Exp clemens $
 * @author  Chr. Clemens Lee
 */
public class StreamCopierTest extends Test 
{
    /**
     * Default test constructor.
     */
    public StreamCopierTest() {
        super();
    }

    /**
     * Standard test constructor.
     */
    public StreamCopierTest( Test pTest_ ) {
        super( pTest_ );
    }

    /**
     * This method contains the test code.
     *
     * @exception   Exception   if anything goes wrong the test fails as well.
     */
    protected void _doIt()
        throws Exception
    {
        {
            // output of one stream is input of another
            PipedOutputStream pos = new PipedOutputStream();
            PrintStream prs = new PrintStream( pos );
            PipedInputStream pis = new PipedInputStream( pos );
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            prs.print( "Hey Jude\n" );
            prs.close();
            Assert( StreamCopier.copy( pis, bos ) == false );
            Assert( bos.toString().equals( "Hey Jude\n" ) );        
        }

        {
            PipedOutputStream pos = new PipedOutputStream();
            PrintStream prs = new PrintStream( pos );
            PipedInputStream pis = new PipedInputStream( pos );
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            prs.print( "Hey Jude\n" );
            prs.close();
            Assert( StreamCopier.copy( pis, bos, 2 ) == false );
            Assert( bos.toString().equals( "y Jude\n" ) );        
        }
    }

    public static void main( String[] asArg_ ) {
        Test pTest = (Test)(new StreamCopierTest());
        pTest.setVerbose( true );
        pTest.run();
        pTest.printResult();

        System.exit( 0 );
    }
}
